/*
 *    colorwin.c:
 *        
 *  First run:  This file is a sample X program designed to
 *  show how to start up a basic X window with a colored background.
 *
 *                 G. "Murdock" Helms, SGI Product Support - 6/11/90        
 */

#include <stdio.h>               /* have to include this to know about NULL */
#include <X11/Xlib.h>            /* Xlib include files */
#include <X11/Xutil.h>           /* Must have these included */

Display *display;
int screen;
/*
   New in this program are the variables gc, valuemask, values, cmap, and
   greenish.

   gc is the graphics context that we will be using, which will help
   us get the correct color we want.  Valuemask is needed for our
   call to create the graphics context, and for now we will leave
   it set to 0.  Values will likewise be used to send to GC, although
   we will not set it.  Cmap will be accepting the value of the
   default colormap that the current window manager is using.  Greenish
   will be used to take the correct values of the color we are looking
   for.
 */

main(argc, argv)
int argc;
char **argv;
{
    Window win, rootwin;
    XSizeHints sizehints;
    char *window_name = "Blue Background";
    char *icon_name = "blue";
    char *display_name = NULL;
    XGCValues values;
    Colormap cmap;               /* we'll use this for the background color */
    XColor greenish;
/* ignore the following two lines until you reach the program queue.c.
   You don't need to worry about these for now. */
    XEvent report;               /* will return our events */
    unsigned long valuemask;     /* null for now */



    if ( (display=XOpenDisplay(display_name)) == NULL ) {
	fprintf( stderr, "%s: cannot connect to X server", argv[0]);
	if (getenv("DISPLAY") == NULL)
            fprintf( stderr, ", 'DISPLAY' environment variable not set.\n");
        else
            fprintf( stderr, " %s\n", XDisplayName(display_name));
        exit ( -1 );
    }

/* Get screen from display structure macro */
    screen = DefaultScreen(display);

    rootwin = RootWindow(display, screen);
    win = XCreateSimpleWindow (display, rootwin, 0, 0, 200, 200, 5,
                               BlackPixel(display, screen),
                               WhitePixel(display, screen));   

/*
   Since we are not creating our own colormap, we will be using the
   same one that our current window manager is using.  To do this,
   we call DefaultColormap and return that value to the variable cmap.
 */

/* set up the colormap for usage */
    cmap=DefaultColormap(display,screen);

/*
   Now we want to take our colormap, and use XParseColor to look up
   the RGB values for our selected color, which is "yellow green",
   and place those values into our variable greenish.  Once we have
   greenish loaded with the right RGB values, we can then use
   XAllocColor to search through our colormap for the RGB
   values in greenish, or failing a match, to come up with the closest
   color to the values in greenish.  Once we have the closest match, we
   then use XSetWindowBackground to the pixel values in greenish.
   Now when the window is drawn, the background will be yellowish green.

 */

/* Attempt to set background color for my window */
    XParseColor(display, cmap, "yellow green", &greenish);
    XAllocColor(display,cmap,&greenish);
    XSetWindowBackground(display, win, greenish.pixel); 

/* Again, ignore the following call.  This is queuing stuff, see
   queue.c for explanations */
    XSelectInput(display, win, ExposureMask | KeyPressMask |
                               ButtonPressMask | StructureNotifyMask);

/* Set properties for window manager (always before mapping!) 
    XSetStandardProperties(display, win, window_name, icon_name, None,
                           0, 0, &sizehints); */

/* Display window */
    XMapWindow(display, win);


/* Get events; use the first Expose to display text and graphics;
 * ConfigureNotify to indicate a resize;  ButtonPress or KeyPress to exit */
    while (1) {

        XNextEvent(display, &report);

        switch (report.type) {

        /* The window has been resized, so change width and height
         * for draw_text and draw_graphics in next Expose */
            case ConfigureNotify:
                break;

        /*Just trickle down into KeyPress (no break) */
            case ButtonPress:
            case KeyPress:

        /* Close down gracefully */
                XCloseDisplay(display);
                exit(1);

        /* All events selected by StructureNotifyMask except ConfigureNotify 
         * are thrown away here, since nothing is done with them */
            default:
                break;

        }          /* end switch */
    }                           /* end while  */
}                                            /* end main   */
