/*
 *    cursor.c:
 *
 *  First run:  This file is a sample X program designed to
 *  show how to start up a basic X window and by allowing you
 *  to change your cursor into another shape, in this case,
 *  everyone's friend Gumby.
 *
 *                 G. "Murdock" Helms, SGI Product Support - 6/11/90	
 */

/*
   We need a new include file to do cursor work with, the file
   cursorfont.h, which contains definitions for the different
   types of cursors.
 */

#include <stdio.h> 	 	 /* have to include this to know about NULL */
#include <X11/Xlib.h>		 /* Xlib include files */
#include <X11/Xutil.h>		 /* Must have these included */
#include <X11/cursorfont.h>	 /* for cursors */

/* Display and screen are used as arguments to nearly every Xlib
 * routine, so it simplifies things to declare them global. */

Display *display;
int screen;

/*
   New in this program is the variable cursor, and cursor_shape.
   We will be using cursor_shape to set the cursor to one of many
   pre-defined cursor types that are listed on pages 641-642 of
   O'Reilly Volume 2.  In this example, we will be using gumby.
   Cursor will be used later to store cursor information.
 */

main(argc, argv)
int argc;
char **argv;
{
    Window win, rootwin;
    XSizeHints sizehints;
    char *window_name = "Gumby Returns";
    char *icon_name = "cursor";
    char *display_name = NULL;
    int x=100,y=100;
    unsigned int width=400, height=400;
    int cursor_shape=XC_gumby;
    Cursor cursor;
/*  Ignore the next two lines.  It has to do with queing up
    the Esc. key.  See queue.doc for more info on this sort of thing.  */
    XEvent report;              /* will return our events */
    unsigned long valuemask;    /* null for now */



/* Connect to  X server ... this is kind of a fancy test to make
 * sure the server is running, but it produces a nicer error code
 * than you normally get. */

    if ( (display=XOpenDisplay(display_name)) == NULL ) {
	fprintf( stderr, "%s: cannot connect to X server", argv[0]);
	if (getenv("DISPLAY") == NULL)
            fprintf( stderr, ", 'DISPLAY' environment variable not set.\n");
        else
            fprintf( stderr, " %s\n", XDisplayName(display_name));
        exit ( -1 );
    }

    /* Get screen from display structure macro */
    screen = DefaultScreen(display);

    /* Create opaque window ... note that BlackPixel and WhitePixel
     * are macros.
     */


    rootwin = RootWindow(display, screen);
    win = XCreateSimpleWindow (display, rootwin, x, y, width, height, 5,
	                       BlackPixel(display, screen),
                               WhitePixel(display, screen));

    sizehints.flags=USPosition;             /* so it can be either/or     */
    sizehints.x=x;                          /* this is part of XSizeHints */
    sizehints.y=y;                          /* see manual on structure of */

    /* Set properties for window manager (always before mapping!)  */
    XSetStandardProperties(display, win, window_name, icon_name, None,
	                   0, 0, &sizehints); 
/*
   We set the cursor by using XCreateFontCursor. We
   are using XCreateFontCursor rather than XCreateGlyphCursor
   or XCreatePixmapCursor because the cursors we are interested
   in are defined as fonts.  XCreateFontCursor then loads information
   about Gumby into our cursor variable.

   Next we use XDefineCursor so that our window knows
   that we want our cursor to be Gumby whenever we move inside it.
   Judicious use of this call in a multiple-window program allows
   different windows to display different cursors.

   In this program, since we do not have an explicit endpoint, we
   then call XFreeCursor so we don't hang up the cursor
   ID after we kill the program.
 */
    
    cursor=XCreateFontCursor(display,cursor_shape);
    XDefineCursor(display,win,cursor);
    XFreeCursor(display,cursor);

/*  More queuing stuff.  Ignore it.  */
    XSelectInput(display, win, ExposureMask | KeyPressMask |
                               ButtonPressMask | StructureNotifyMask);

    /* Display window */
    XMapWindow(display, win);

/* Get events; use the first Expose to display text and graphics;
 * ConfigureNotify to indicate a resize;  ButtonPress or KeyPress to exit */
    while (1) {

        XNextEvent(display, &report);

        switch (report.type) {

        /*Just trickle down into KeyPress (no break) */
            case ButtonPress:
            case KeyPress:

        /* Close down gracefully */
                XCloseDisplay(display);
                exit(1);

        /* All events selected by StructureNotifyMask except ConfigureNotify 
	 * are thrown away here, since nothing is done with them */
            default:
                break;

        }          /* end switch */
    }                           /* end while  */
}                  	                     /* end main   */
