/*        
 *    font.c: 
 *
 *  First run:  This file is a sample X program designed to show how 
 *  to use fonts and print things in a window.
 *
 *                 G. "Murdock" Helms, SGI Product Support - 6/11/90
 */
/*
   The include files in font.c have a new addition to the ones
   listed in placewin.  The new include file is string.h, which
   enables us to pass a font string to the window.  Also new in
   this program is the changing of the win variable, formerly a
   local variable in main, to be a global variable.

 */

#include <stdio.h>               /* have to include this to know about NULL */
#include <string.h>
#include <X11/Xlib.h>            /* Xlib include files */
#include <X11/Xutil.h>           /* Must have these included */

/* Display and screen are used as arguments to nearly every Xlib
 * routine, so it simplifies things to declare them global. */

Display *display;
int screen;
Window win;

/*
   We have two new variables declared in main, which are
   *font_info, and gc.  *font_info is of type XFontStruct,
   and we will be assigning properties of our chosen font
   to it later in the program.  gc is of type GC, or graphics
   context.  Many of the X font calls require the passage
   of a graphics context, so even though we are not using
   graphics per se in this program, we still need to declare
   a gc.
 */

main(argc,argv)
int argc;
char **argv;
{
    Window rootwin;
    XSizeHints sizehints;
    char *window_name = "Another Font";
    char *icon_name = "font";
    char *display_name = NULL;
    int x=100,y=100;
    unsigned int width=400, height=400;
    XFontStruct *font_info;                     /* needed for font stuff */
    GC gc;                                      /* needed to do text output */
/*  Ignore the next two lines.  It has to do with queing up
    the Esc. key.  See queue.doc for more info on this sort of thing.  */
    XEvent report;                              /* will return our events */
    unsigned long valuemask;                    /* null for now */



/* Connect to  X server ... this is kind of a fancy test to make
 * sure the server is running, but it produces a nicer error code
 * than you normally get. */

    if ( (display=XOpenDisplay(display_name)) == NULL ) {
	fprintf( stderr, "%s: cannot connect to X server", argv[0]);
	if (getenv("DISPLAY") == NULL)
            fprintf( stderr, ", 'DISPLAY' environment variable not set.\n");
        else
            fprintf( stderr, " %s\n", XDisplayName(display_name));
        exit ( -1 );
    }
    
/* Get screen from display structure macro */
    screen = DefaultScreen(display);

/* Create opaque window ... note that BlackPixel and WhitePixel
 * are macros.
 */
    rootwin = RootWindow(display, screen);
    win = XCreateSimpleWindow (display, rootwin, x, y, width, height, 5,
                               BlackPixel(display, screen),
                               WhitePixel(display, screen));

/* Check that the font you want is actually there and set it
 * to something useable. (SEE BELOW )*/
   load_my_font(&font_info);

/* Still need a GC for fonts(SEE BELOW ) */
   get_GC(win,&gc,font_info);

/* Set properties for window manager (always before mapping!)  */
    XSetStandardProperties(display, win, window_name, icon_name, None,
                           0, 0, &sizehints); 


/*  More queuing stuff.  Ignore it.  */
    XSelectInput(display, win, ExposureMask | KeyPressMask |
                               ButtonPressMask | StructureNotifyMask);
    
/* Display window */
    XMapWindow(display, win);

/* Print out text stuff (SEE BELOW )*/
   draw_text(win, gc, font_info);

/* Get events; use the first Expose to display text and graphics;
 * ConfigureNotify to indicate a resize;  ButtonPress or KeyPress to exit */
    while (1) {

        XNextEvent(display, &report);

        switch (report.type) {

	/* Draw our text first */
	case Expose:
		draw_text(win, gc, font_info);
		break;

        /*Just trickle down into KeyPress (no break) */
            case ButtonPress:
            case KeyPress:

        /* Close down gracefully */
                XCloseDisplay(display);
                exit(1);

        /* All events selected by StructureNotifyMask except ConfigureNotify 
         * are thrown away here, since nothing is done with them */
            default:
                break;

        }          /* end switch */
    }                           /* end while  */
}                                            /* end main   */

/*
  After creating a window, we want to check on the fonts
  and set up the information about the font so we can use it.
  In load_my_font we set our chosen font type to
  "olglyph-19", using the variable *fontname.  Then we use
  XLoadQueryFont to see if olglyph is available on the machine
  it will be running on.  If olglyph is available, XLoadQueryFont
  will go ahead and load up font_info with the appropriate font
  information.

  If olglyph-19 is not available, we want our program to be a
  polite program, and it informs the user that the font is not
  there, and it will use 9x15 font instead (a common enough font
  so it shouldn't cause any trouble).  At that point, we load
  up font_info with information on 9x15 by using XLoadQueryFont.
 */

load_my_font(font_info)
XFontStruct **font_info;
{
   char *fontname = "9x15";


/*  Access the font  */
   if ((*font_info = XLoadQueryFont(display,fontname)) == NULL)
   {
        (void) fprintf(stderr,"Sorry,can't find that font.  Using 9x15 instead.\n"); 

/*  If it can't be found, be polite and set it to something
 *  else so the program doesn't just puke */
        fontname="9x15";
        *font_info = XLoadQueryFont(display,fontname);
   } 
}



/*
  Now that we have our font, we have to worry about setting
  up the graphics context with that font information.  In
  get_GC we go through that procedure.

  We have two new variables to use with the GC calls, valuemask
  and values.  XCreateGC requires these variable, but since we will
  not be using valuemask or values in this program, we set valuemask
  to 0, and leave values empty.  XCreateGC will then pass to gc
  a new graphics context.

  Now that we've created the graphics context, we can use XSetFont
  to set the font, and we also use XSetForeground to make certain
  that the font will appear in black in our window.
 */

get_GC(win,gc,font_info)
Window win;
GC *gc;
XFontStruct *font_info;
{
   unsigned long valuemask = 0;
   XGCValues values;


/* Create a default graphics context */
   *gc=XCreateGC(display,win,valuemask,&values);

/* Specify the font */
   XSetFont(display, *gc,font_info->fid); 
   XSetForeground(display, *gc, BlackPixel(display,screen));
}

/*
   After setting up the font information correctly, we can go ahead
   and set the window properties and map the window.  After mapping,
   we then use draw_text (line 113) to actually write out the string.
   *string1 contains the string we want to write, and len1 will be
   set to the length of that string as calculated by strlen. XDrawString
   will then take the string itself and it's length to draw the string
   in the window.
 */

draw_text(win,gc,font_info)
Window win;
GC gc;
XFontStruct *font_info;
{
   char *string1 = "A different kind of font";
   char *string2 = "Press any key to quit.";
   int len1,len2;


/* need length for DrawString */
   len1=strlen(string1);
   len2=strlen(string2);

/*Output text centered in window width */
   XDrawString(display,win,gc,30,30,string1,len1);
   XDrawString(display,win,gc,30,60,string2,len1);
}
