/* 
 *    queue.c:
 *        
 *  First run:  This file is a sample X program designed to
 *  show how to use queueing with a window
 *
 *                 G. "Murdock" Helms, SGI Product Support - 10/11/90
 */

#include <stdio.h>               /* have to include this to know about NULL */
#include <string.h>
#include <X11/Xlib.h>            /* Xlib include files */
#include <X11/Xutil.h>           /* Must have these included */

/* Display and screen are used as arguments to nearly every Xlib
 * routine, so it simplifies things to declare them global. */

Display *display;
int screen;
Window win;
/*
   New in this program are report and valuemask.  Report will be
   used to tell us which event has just occurred, while valuemask
   will be pretty much ignored.  We need valuemask to use when we
   work with graphics contexts, but we won't be using it here.
 */

main(argc,argv)
int argc;
char **argv;
{
    Window rootwin;
    XSizeHints sizehints;
    char *window_name = "Queue Program";
    char *icon_name = "Queue";
    char *display_name = NULL;
    int x=100,y=100;
    unsigned int width=400, height=400;
    XFontStruct *font_info;         
    GC gc;                        
    XEvent report;                /* will return our events */
    unsigned long valuemask;      /* null for now */



/* Connect to  X server ... this is kind of a fancy test to make
 * sure the server is running, but it produces a nicer error code
 * than you normally get. */

    if ( (display=XOpenDisplay(display_name)) == NULL ) {
	fprintf( stderr, "%s: cannot connect to X server", argv[0]);
	if (getenv("DISPLAY") == NULL)
            fprintf( stderr, ", 'DISPLAY' environment variable not set.\n");
        else
            fprintf( stderr, " %s\n", XDisplayName(display_name));
        exit ( -1 );
    }
    
/* Get screen from display structure macro */
    screen = DefaultScreen(display);

/* Create opaque window ... note that BlackPixel and WhitePixel
 * are macros.
 */
    rootwin = RootWindow(display, screen);
    win = XCreateSimpleWindow (display, rootwin, x, y, width, height, 5,
                               BlackPixel(display, screen),
                               WhitePixel(display, screen));

/* Check that the font you want is actually there and set it
 * to something useable. */
   load_my_font(&font_info);

/* Still need a GC for fonts */
   get_GC(win,&gc,font_info);

/* Set properties for window manager (always before mapping!)  */
    XSetStandardProperties(display, win, window_name, icon_name, None,
                           0, 0, &sizehints); 
/*
   We need to tell the window manager what types of events we're
   interested in, so we use XSelectInput to state that
   we are interested in exposures, key presses, (mouse) button
   presses, or any kind of circulation (pushing or popping) or
   destruction events.
 */

/* Select event types wanted */
    XSelectInput(display, win, ExposureMask | KeyPressMask |
                               ButtonPressMask | StructureNotifyMask);
    
/* Display window */
    XMapWindow(display, win);

/* Print out text stuff */
   draw_text(win, gc, font_info);
/*
   Instead of using a while (1) loop through infinity, we replace
   it with a loop that will use XNextEvent to grab the
   next event it sees, then pass the resulting event (in report)
   through a series of checks.  If the event is of type Configure
   Notify, it actually means that something has happened that is
   of type StructureNotifyMask.  That is, ConfigureNotify events
   tells us that the window has changed sizes, position, stacking
   order, etc.  We are interested in events type ConfigureNotify
   because we will probably need to redraw the window for one
   reason or another.  In this case, we get the new size and re-
   draw our text.

   If instead the event is a ButtonPress or a KeyPress, we use
   XCloseDisplay to close the window gracefully.  Any other events
   are ignored.
 */

/* Get events; use the first Expose to display text and graphics;
 * ConfigureNotify to indicate a resize;  ButtonPress or KeyPress to exit */
    while (1) {

        XNextEvent(display, &report);

        switch (report.type) {

	/* Draw our text first */
	    case Expose:
		draw_text(win, gc, font_info);
		break;

        /* The window has been resized, so change width and height
         * for draw_text and draw_graphics in next Expose */
            case ConfigureNotify:
                width = report.xconfigure.width;
                height = report.xconfigure.height;
                draw_text(win, gc, font_info);
                break;

        /*Just trickle down into KeyPress (no break) */
            case ButtonPress:
            case KeyPress:

        /* Close down gracefully */
                XCloseDisplay(display);
                exit(1);

        /* All events selected by StructureNotifyMask except ConfigureNotify 
         * are thrown away here, since nothing is done with them */
            default:
                break;

        }          /* end switch */
    }                           /* end while  */
}                                            /* end main   */



load_my_font(font_info)
XFontStruct **font_info;
{
   char *fontname = "iris10";


/*  Access the font  */
   if ((*font_info = XLoadQueryFont(display,fontname)) == NULL)
   {
        (void) fprintf(stderr,"Sorry,can't find that font.  Using 9x15 instead.\n"); 

/*  If it can't be found, be polite and set it to something
    else so the program doesn't just puke */
        fontname="9x15";
        *font_info = XLoadQueryFont(display,fontname);
   } 
}



get_GC(win,gc,font_info)
Window win;
GC *gc;
XFontStruct *font_info;
{
   unsigned long valuemask = 0;
   XGCValues values;


/* Create a default graphics context */
   *gc=XCreateGC(display,win,valuemask,&values);

/* Specify the font */
   XSetFont(display, *gc,font_info->fid); 
   XSetForeground(display, *gc, BlackPixel(display,screen));
}



draw_text(win,gc,font_info)
Window win;
GC gc;
XFontStruct *font_info;
{
   char *string1 = "Press Any Key!";
   int len1;


/* need length for DrawString */
   len1=strlen(string1);

/*Output text centered in window width */
   XDrawString(display,win,gc,30,30,string1,len1);
}
