/*
 *    simplewin.c:
 * 
 *  First run:  This file is a sample X program designed to show how to 
 *  start up the most basic window going.  That's all it does, too.
 *
 *                 G. "Murdock" Helms, SGI Product Support - 5/8/90
 */


/*
      First of all we have the include files.  An X program requires at
    a minimum inclusion of the Xlib.h and Xutil.h header files (both live
    in /usr/include/X11).  Inclusion of stdio.h is also necessary if you
    intend to use the "NULL" constant.
 */

#include <stdio.h>               /* have to include this to know about NULL */
#include <X11/Xlib.h>            /* Xlib include files */
#include <X11/Xutil.h>           /* Must have these included */

/*
      Next, we declare the variables "display" and "screen" as globals.
    Since these values are referenced in nearly every single X call in
    this program, it seemed to be easier to make them globals than to
    keep re-declaring them.

 */

Display *display;
int screen;

main(argc,argv)
int argc;
char **argv;
{
/*
      Both "win" and "rootwin" are used to determine the child window id
    and parent window id, respectively.   Their values are determined by
    the output of functions we will examine later.

      "window_name" and "icon_name" are used later when we set properties
    for the window, but we go ahead and initialize them now.  However,
    "display_name" is initially set to NULL so we can establish whether
    the X server is running when we use it in the XOpenDisplay call (see
    later).
 */
    Window win, rootwin;        /* both of these lines get set later */
    static XSizeHints size_hints ;
    char *window_name = "Your Basic Window Program";
    char *icon_name = "bwin";
    char *display_name = NULL;  /* since we haven't set anything yet */

/*  Ignore the next two lines.  It has to do with queing up
    the Esc. key.  See queue.doc for more info on this sort of thing.  */
    XEvent report;              /* will return our events */
    unsigned long valuemask;    /* null for now */


/*
       XOpenDisplay() will first look for a coded "display_name",
    and if it can't find one, will then examine the environment variable
    "DISPLAY" to see if it is defined.  If "DISPLAY" is not defined, the
    user needs to realize that all is not kosher to correctly execute her
    program, and we should tell her so before exiting.

 */

/* Connect to  X server ... this is kind of a fancy test to make
 * sure the server is running, but it produces a nicer error code
 * than you normally get.
 */

    if ( (display=XOpenDisplay(display_name)) == NULL ) {
	fprintf( stderr, "%s: cannot connect to X server", argv[0]);
	if (getenv("DISPLAY") == NULL)
            fprintf( stderr, ", 'DISPLAY' environment variable not set.\n");
        else
            fprintf( stderr, " %s\n", XDisplayName(display_name));
        exit ( -1 );
    }

/*
      Next  we assign "screen" the returned value of
    DefaultScreen(), the macro which returns the default screen number
    referenced above with "display" in the XOpenDisplay() routine.
    Typically, the returned value is 0 (for displaying the window on the
    local machine).
 */
    /* Get screen from display structure macro */
    screen = DefaultScreen(display);

/*
      Then we assign "rootwin" the returned value of the
    RootWindow() macro.  This macro returns the root (parent) window ID
    which is needed to correctly call XCreateSimpleWindow(), the next
    routine.  In its turn, XCreateSimpleWindow() returns the ID of the
    window it has created in the current display, which is a child of the
    root window specified.  This child window has an upper-left corner
    [which should be] located at coordinates 100,100, is 200 pixels tall
    and 150 pixels wide, has a border width of 5, and a border color of
    black with a white background inside the window.
 */
    /* Create opaque window ... note that BlackPixel and WhitePixel
     * are macros.  */

    rootwin = RootWindow(display, screen);
    win = XCreateSimpleWindow (display, rootwin, 100, 100, 200, 150, 5,
                               BlackPixel(display, screen),
                               WhitePixel(display, screen));

/*
      At this point, we take the window id returned by
    XCreateSimpleWindow() and set some standard properties, such as the
    window name, icon name, any icon pixmaps, arguments, and hints for
    sizes that the window may have.  In this particular case,
    even though we have defined the window to be of a certain size, we are
    not working with size hints.  This means the window manager may decide
    on its own discretion whether or not to obey the size info we have
    given it during its creation.
 */

    /* Set properties for window manager before mapping it */
    XSetStandardProperties(display, win, window_name, icon_name, None,
                           0, 0, &size_hints);

/*  More queuing stuff.  Ignore it.  */
    XSelectInput(display, win, ExposureMask | KeyPressMask |
                               ButtonPressMask | StructureNotifyMask);
/*
      XMapWindow() will now make the window "eligible for display";  that
    is, we are now ready to display it, but it may need more coaxing to
    actually show up on our graphics console screen.  To this end, we employ
    XFlush(), which flushes the output buffer.  There are three ways to
    flush this buffer:  by calling XFlush(), by using XSync(), or to check
    the contents (if any) of the event queue.  We won't be using the event
    queue until later on, so at this point we can simply call XFlush() to
    see the window we have so meticulously created here.
 */    
    /* Put window up */
    XMapWindow(display, win);

/*
      To quit this window/end this program you will need to select "Quit"
    from the window menu, or hitting delete in
    the shell window you invoked the program from in the first place.  Any
    of these methods will still result in your shell appearing to be hung.
    Hitting delete should return your shell prompt to you.
    In fact any key or button press will also work.
 */

/* Get events; use the first Expose to display text and graphics;
 * ConfigureNotify to indicate a resize;  ButtonPress or KeyPress to exit */
    while (1) {

        XNextEvent(display, &report);

        switch (report.type) {

            /*Just trickle down into KeyPress (no break) */
            case ButtonPress:
            case KeyPress:

            /* Close down gracefully */
                XCloseDisplay(display);
                exit(1);        

            /* All events selected by StructureNotifyMask except
             * ConfigureNotify are thrown away here, since
             * nothing is done with them */
            default:
                break;

        }          /* end switch */
    }                           /* end while  */
}                                            /* end main   */
